@@ -103,11 +103,15 @@ struct _ply_boot_splash_plugin
   uint32_t root_is_mounted : 1;
   uint32_t is_visible : 1;
   uint32_t is_animating : 1;
+  uint32_t is_idle : 1;
 };
 
 ply_boot_splash_plugin_interface_t * ply_boot_splash_plugin_get_interface (void);
 static void detach_from_event_loop (ply_boot_splash_plugin_t *plugin);
 
+static void become_idle (ply_boot_splash_plugin_t *plugin,
+                         ply_trigger_t            *idle_trigger);
+
 static view_t *
 view_new (ply_boot_splash_plugin_t *plugin,
           ply_pixel_display_t      *display)
@@ -124,8 +128,6 @@ view_new (ply_boot_splash_plugin_t *plugin,
   view->progress_bar = ply_progress_bar_new ();
   view->label = ply_label_new ();
   view->message_label = ply_label_new ();
-  ply_label_set_text (view->message_label, "");
-  ply_label_show (view->message_label, view->display, 10, 10);
 
   return view;
 }
@@ -304,6 +306,8 @@ view_start_animation (view_t *view)
   if (plugin->mode == PLY_BOOT_SPLASH_MODE_SHUTDOWN)
     return;
 
+  plugin->is_idle = false;
+
   width = ply_throbber_get_width (view->throbber);
   height = ply_throbber_get_height (view->throbber);
   ply_throbber_start (view->throbber,
@@ -521,6 +525,9 @@ start_animation (ply_boot_splash_plugin_t *plugin)
     }
 
   plugin->is_animating = true;
+
+  if (plugin->mode == PLY_BOOT_SPLASH_MODE_SHUTDOWN)
+    plugin->is_idle = true;
 }
 
 static void
@@ -706,6 +713,9 @@ on_boot_progress (ply_boot_splash_plugin_t *plugin,
   ply_list_node_t *node;
   double total_duration;
 
+  if (plugin->mode == PLY_BOOT_SPLASH_MODE_UPDATES)
+    return;
+
   total_duration = duration / percent_done;
 
   /* Fun made-up smoothing function to make the growth asymptotic:
@@ -804,7 +814,14 @@ static void
 become_idle (ply_boot_splash_plugin_t *plugin,
              ply_trigger_t            *idle_trigger)
 {
+  if (plugin->is_idle)
+    {
+      ply_trigger_pull (idle_trigger, NULL);
+      return;
+    }
+
   stop_animation (plugin, idle_trigger);
+  plugin->is_idle = true;
 }
 
 static void
@@ -842,6 +859,8 @@ show_message (ply_boot_splash_plugin_t *plugin,
       view = ply_list_node_get_data (node);
       next_node = ply_list_get_next_node (plugin->views, node);
       ply_label_set_text (view->message_label, message);
+      ply_label_show (view->message_label, view->display, 10, 10);
+
       ply_pixel_display_draw_area (view->display, 10, 10,
                                    ply_label_get_width (view->message_label),
                                    ply_label_get_height(view->message_label));
@@ -899,6 +918,29 @@ display_message (ply_boot_splash_plugin_t *plugin,
   show_message (plugin, message);
 }
 
+static void
+system_update (ply_boot_splash_plugin_t *plugin,
+               int                       progress)
+{
+  ply_list_node_t *node;
+
+  if (plugin->mode != PLY_BOOT_SPLASH_MODE_UPDATES)
+    return;
+
+  node = ply_list_get_first_node (plugin->views);
+  while (node != NULL)
+    {
+      ply_list_node_t *next_node;
+      view_t *view;
+
+      view = ply_list_node_get_data (node);
+      next_node = ply_list_get_next_node (plugin->views, node);
+      ply_progress_bar_set_percent_done (view->progress_bar, (double) progress / 100.f);
+      ply_progress_bar_draw (view->progress_bar);
+      node = next_node;
+    }
+}
+
 ply_boot_splash_plugin_interface_t *
 ply_boot_splash_plugin_get_interface (void)
 {
@@ -918,6 +960,7 @@ ply_boot_splash_plugin_get_interface (void)
       .display_password = display_password,
       .display_question = display_question,      
       .display_message = display_message,
+      .system_update = system_update,
     };
 
   return &plugin_interface;
